// Extension Preferences Menu
// Author: Antonius32
// Version 2.13.14 (March 2016)

var ExtensionPreferencesMenu = {
  startup: function() {
    window.removeEventListener("load", function EPMload() { ExtensionPreferencesMenu.startup() });

    window.addEventListener("customizationchange", function EPMcustomizationchange() { ExtensionPreferencesMenu.setup() });

    Components.utils.import("resource://gre/modules/AddonManager.jsm");
    AddonManager.addAddonListener(this);

    Components.utils.import("resource://gre/modules/Services.jsm");
    this._branch = Services.prefs.getBranch("extensions.ExtensionPreferencesMenu.");
    this._branch.addObserver("", this, false);

    // Delete preferences that have been renamed, after transferring their values
    if (this._branch.prefHasUserValue("ToolsMenu_ShowIcon")) {
      let EPMoldValue = this._branch.getBoolPref("ToolsMenu_ShowIcon");
      this._branch.setBoolPref("IconInToolsMenu", EPMoldValue);
      this._branch.clearUserPref("ToolsMenu_ShowIcon");
    }
    if (this._branch.prefHasUserValue("VersionNumbers_HideSigned")) {
      let EPMoldValue = this._branch.getBoolPref("VersionNumbers_HideSigned");
      this._branch.setBoolPref("HideSigned", EPMoldValue);
      this._branch.clearUserPref("VersionNumbers_HideSigned");
    }
    if (this._branch.prefHasUserValue("BlackList")) {
      let EPMoldValue = this._branch.getCharPref("BlackList");
      this._branch.setCharPref("Blacklist", EPMoldValue);
      this._branch.clearUserPref("BlackList");
    }

    // Delete obsolete preference
    if (this._branch.prefHasUserValue("ToolsMenu_ReplaceOriginal"))
      this._branch.clearUserPref("ToolsMenu_ReplaceOriginal");
    if (this._branch.prefHasUserValue("OpenExtensionsPage"))
      this._branch.clearUserPref("OpenExtensionsPage");

    if (this._branch.getBoolPref("FirstRun"))
      this.onFirstRun();

    this.setup();
  },

  onEnabling: function() this.setup(),

  onEnabled: function() this.setup(),

  onDisabling: function() this.setup(),

  onDisabled: function() this.setup(),

  onInstalled: function() this.setup(),

  onUninstalled: function() this.setup(),

  onOperationCancelled: function() this.setup(),

  observe: function(aSubject, aTopic, aData) {
    if (aTopic == "nsPref:changed")
      this.setup();
  },

  onFirstRun: function() {
    this._branch.setBoolPref("FirstRun", false);

    // Pale Moon
    let navBar = document.getElementById("nav-bar");
    if (navBar) {
      // Add the toolbar button to the Navigation toolbar
      let downloadsButton = document.getElementById("downloads-button");
      navBar.contains(downloadsButton) ? navBar.insertItem("extensionPrefsMenu-button", downloadsButton.nextSibling) :
                                         navBar.insertItem("extensionPrefsMenu-button");
      navBar.setAttribute("currentset", navBar.currentSet);
      document.persist("nav-bar", "currentset");

      // Make sure the Navigation toolbar is visible
      if (navBar.collapsed) {
        navBar.collapsed = false;
        document.persist("nav-bar", "collapsed");
      }
    }

    // FossaMail
    let mailBar = document.getElementById("mail-bar3");
    if (mailBar) {
      // Add the toolbar button to the Mail toolbar
      let globalSearchBox = document.getElementById("gloda-search");
      mailBar.contains(globalSearchBox) ? mailBar.insertItem("extensionPrefsMenu-button", globalSearchBox.nextSibling) :
                                          mailBar.insertItem("extensionPrefsMenu-button");
      mailBar.setAttribute("currentset", mailBar.currentSet);
      document.persist("mail-bar3", "currentset");

      // Make sure the Mail toolbar is visible
      if (mailBar.collapsed) {
        mailBar.collapsed = false;
        document.persist("mail-bar3", "collapsed");
      }
    }
  },

  setup: function() {
    let EPMappMenu = document.getElementById("appmenu_extensionPrefs");
    let origAppMenu = document.getElementById("appmenu_addons");
    EPMappMenu.collapsed = !this._branch.getBoolPref("InAppMenu");
    origAppMenu.collapsed = !EPMappMenu.collapsed;

    this.populate(EPMappMenu.firstChild);

    let EPMtoolsMenu = document.getElementById("extensionPrefsMenu");
    EPMtoolsMenu.collapsed = !this._branch.getBoolPref("InToolsMenu");

    if (this._branch.getBoolPref("IconInToolsMenu"))
      EPMtoolsMenu.setAttribute("style", "list-style-image: url(chrome://mozapps/skin/extensions/extensionGeneric-16.png)");
    else
      EPMtoolsMenu.setAttribute("style", "list-style-image: none");

    this.populate(EPMtoolsMenu.firstChild);

    let EPMtoolbarButton = document.getElementById("extensionPrefsMenu-button");
    if (EPMtoolbarButton)
      this.populate(EPMtoolbarButton.firstChild);
  },

  populate: function(aParent) {
    let appMenuPopup = document.getElementById("appmenu-popup");
    let toolsMenuPopup = document.getElementById("menu_ToolsPopup") || document.getElementById("taskPopup");
    let EPMappPopup = document.getElementById("EPMAppPopup");
    let EPMtoolsPopup = document.getElementById("EPMToolsPopup");
    let EPMbuttonPopup = document.getElementById("EPMButtonPopup");
    let item, menuIconURL, EPMshowIt = true, EPMblacklistArray = [];
    let EPMshowVersionNumbers = this._branch.getBoolPref("ShowVersionNumbers");
    let EPMshowIcons = this._branch.getBoolPref("ShowIcons");
    let EPMshowAll = this._branch.getBoolPref("ShowAll");
    let EPMshowDisabled = this._branch.getBoolPref("ShowDisabled");
    let EPMshowIncompatible = this._branch.getBoolPref("ShowIncompatible");
    let EPMquickActions = this._branch.getBoolPref("QuickActions");
    let EPMaddonsManagerLabel = document.getElementById("EPMStringBundle").getString("EPMAddonsManager");
    let EPMshowRestart = this._branch.getBoolPref("ShowRestart");
    let EPMrestartLabel = document.getElementById("EPMStringBundle").getString("EPMRestart");
    let EPMopenApp = window["BrowserOpenAddonsMgr"] || window["openAddonsMgr"];
    let EPMscrollInToView = this._branch.getBoolPref("ScrollInToView");
    let EPMhideSigned = this._branch.getBoolPref("HideSigned");
    let EPMblacklist = this._branch.getCharPref("Blacklist");
    if (EPMblacklist) {
      EPMblacklist = EPMblacklist.split("").reverse().join("");
      EPMblacklistArray = EPMblacklist.split(/\,(?!\\)/);
      for (let i = 0; i < EPMblacklistArray.length; i++)
        EPMblacklistArray[i] = EPMblacklistArray[i].split("").reverse().join("").replace(/^ | $/g,"").replace(/\\\,/g,",").toLowerCase();
    }

    AddonManager.getAllAddons(function(addon) {
      while (aParent.childNodes.length)
        aParent.removeChild(aParent.firstChild);

      addon.forEach(function(EPMaddon) {
        if (EPMaddon.type == "extension" && (!EPMaddon.appDisabled || (EPMaddon.appDisabled && EPMshowIncompatible)) &&
            ((EPMaddon.isActive && EPMaddon.optionsURL) || ((EPMaddon.userDisabled && EPMshowDisabled) ||
             (!EPMaddon.userDisabled && EPMshowAll) || (EPMaddon.appDisabled && EPMshowIncompatible)))) {
          item = document.createElement("menuitem");
          item.setAttribute("closemenu", "none");
          item.setAttribute("label", EPMshowVersionNumbers ? EPMaddon.name + "  " +
                            (EPMhideSigned ? EPMaddon.version.replace(".1-signed", ".1") : EPMaddon.version) : EPMaddon.name);
          if (EPMaddon.description)
            item.setAttribute("tooltiptext", EPMaddon.description);

          item.addEventListener("command", function(event) {
            if (EPMaddon.optionsURL && !event.ctrlKey) {
              if (aParent == EPMappPopup)
                appMenuPopup.hidePopup();
              else if (aParent == EPMtoolsPopup)
                toolsMenuPopup.hidePopup();
              else
                EPMbuttonPopup.hidePopup();
              switch (Number(EPMaddon.optionsType)) {
              case 2:
                EPMopenApp("addons://detail/" + encodeURIComponent(EPMaddon.id) + (EPMscrollInToView ? "/preferences" : ""));
                break;
              case 3:
                switchToTabHavingURI(EPMaddon.optionsURL, true);
                break;
              default:
                let windows = Services.wm.getEnumerator(null);
                while (windows.hasMoreElements()) {
                  let win = windows.getNext();
                  if (win.document.documentURI == EPMaddon.optionsURL) {
                    win.focus();
                    return;
                  }
                }
                let instantApply = Application.prefs.get("browser.preferences.instantApply");
                let features = "chrome,titlebar,toolbar,centerscreen" + (instantApply.value ? ",dialog=no" : ",modal");
                openDialog(EPMaddon.optionsURL, EPMaddon.name, features);
              }
            }
          });

          item.addEventListener("click", function(event) {
            if (EPMquickActions) {
              if (event.button == 1 && EPMaddon.homepageURL) {
                if ("gBrowser" in window) { // Pale Moon
                  if (aParent == EPMappPopup)
                    appMenuPopup.hidePopup();
                  else if (aParent == EPMtoolsPopup)
                    toolsMenuPopup.hidePopup();
                  else
                    EPMbuttonPopup.hidePopup();
                  openURL(EPMaddon.homepageURL);
                }
                else // FossaMail
                  Cc["@mozilla.org/messenger;1"].createInstance(Ci.nsIMessenger).launchExternalURL(EPMaddon.homepageURL);
              }
              else if (event.button == 2 || (event.button == 0 && event.ctrlKey)) {
                AddonManager.getAddonByID(EPMaddon.id, function(addon) {
                  addon.userDisabled = !addon.userDisabled;
                  if (addon.operationsRequiringRestart && !EPMshowRestart) {
                    Application.restart();
                  }
                });
              }
            }
          });

          if (!EPMaddon.optionsURL)
            item.setAttribute("disabled", true);
          if (!EPMaddon.isActive)
            item.setAttribute("EPMdisabled", true);
          if (EPMaddon.pendingOperations == 1)
            item.setAttribute("EPMpendingAction", "enable");
          if (EPMaddon.pendingOperations == 2)
            item.setAttribute("EPMpendingAction", "disable");

          if (EPMshowIcons) {
            item.setAttribute("class", "menuitem-iconic");
            menuIconURL = EPMaddon.iconURL || "chrome://mozapps/skin/extensions/extensionGeneric-16.png";
            item.setAttribute("style", "list-style-image: url(" + menuIconURL + ")");
          }
          else
            item.setAttribute("class", "menuitem");

          if (EPMblacklistArray)
            for (let i = 0; i < EPMblacklistArray.length; i++)
              if (EPMblacklistArray[i] == EPMaddon.name.toLowerCase()) {
                EPMshowIt = false;
                break;
              }
              else
                EPMshowIt = true;

          if (EPMshowIt)
            aParent.appendChild(item);
        }
      });

      Cc["@mozilla.org/xul/xul-sort-service;1"].getService(Ci.nsIXULSortService).sort(aParent, "label", "ascending");

      if (aParent == EPMappPopup) {
        aParent.insertBefore(document.createElement("menuseparator"), aParent.firstChild);

        item = document.createElement("menuitem");
        item.setAttribute("label", EPMaddonsManagerLabel);

        if ("gBrowser" in window) { // Pale Moon
          item.setAttribute("key", "key_openAddons");
          item.addEventListener("command", function() { BrowserOpenAddonsMgr() });
        }
        else // FossaMail
          item.addEventListener("command", function() { openAddonsMgr() });

        if (EPMshowIcons) {
          item.setAttribute("class", "menuitem-iconic");
          item.setAttribute("style", "list-style-image: url(chrome://mozapps/skin/extensions/extensionGeneric-16.png)");
        }
        else
          item.setAttribute("class", "menuitem");

        aParent.insertBefore(item, aParent.firstChild);
      }

      if (EPMshowRestart) {
        aParent.appendChild(document.createElement("menuseparator"));

        item = document.createElement("menuitem");
        item.setAttribute("label", EPMrestartLabel);
        item.addEventListener("command", function() { Application.restart() });

        if (EPMshowIcons) {
          item.setAttribute("class", "menuitem-iconic");
          item.setAttribute("style", "list-style-image: url(chrome://prefsmenu/skin/restart.png)");
        }
        else
          item.setAttribute("class", "menuitem");

        aParent.appendChild(item);
      }
    });
  },

  shutdown: function() {
    window.removeEventListener("unload", function EPMunload() { ExtensionPreferencesMenu.shutdown() });
    AddonManager.removeAddonListener(this);
    this._branch.removeObserver("", this);
    window.removeEventListener("customizationchange", function EPMcustomizationchange() { ExtensionPreferencesMenu.setup() });
  }
};

window.addEventListener("load", function EPMload() { ExtensionPreferencesMenu.startup() });
window.addEventListener("unload", function EPMunload() { ExtensionPreferencesMenu.shutdown() });
